using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using PhoenixDLL.Class.Managers;
using PhoenixDLL.GUI.Seekers;
using PhoenixDLL.GUI;
using Utils.DataBase;
using PhoenixDLL.Class;
using PhoenixDLL.GUI.Managers;
using PhoenixDLL.Print;
using System.Data.OleDb;

namespace PhoenixPrescription.GUI
{
    /// <summary>
    /// Fentre principale des prescriptions
    /// </summary>
    public partial class fr_Main : Form
    {
        #region Variables
        /// <summary>
        /// Manager de patient
        /// </summary>
        private PatientManager vPatientManager = new PatientManager();
        /// <summary>
        /// Gestionnaire de prescription
        /// </summary>
        private PrescriptionManager vPrescriptionManager = new PrescriptionManager();
        /// <summary>
        /// Liste des mdicament prescrit cette session
        /// </summary>
        private List<PrescriptionManager> vPrescriptionList = new List<PrescriptionManager>();
        /// <summary>
        /// Gestionnair de requte SQL
        /// </summary>
        private OleDbSql SqlManager = new OleDbSql(Options.GetInstance().Path);
        /// <summary>
        /// Gestionnaire d'impression de prescriptions
        /// </summary>
        private PrescriptionPrint vPrescriptionPrint = new PrescriptionPrint();
        #endregion

        #region Constructeur
        /// <summary>
        /// Constructeur
        /// </summary>
        public fr_Main()
        {
            InitializeComponent();
            SetLabelsVisible(false);
            lbl_Medicament.Text = "";

            #region Visibilit des menus
            tsMenu_Managment.Checked = Options.Instance.PegaMenu.ManagmentVisible;
            ts_Managment.Visible = Options.Instance.PegaMenu.ManagmentVisible;
            #endregion
        }
        #endregion

        #region Event
        /// <summary>
        /// Clique sur le menu d'options
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_Options(object sender, EventArgs e)
        {
            fr_Options fr = new fr_Options();
            fr.ShowDialog();
        }
        /// <summary>
        /// Clique sur selection d'un patient
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_SelectPatient(object sender, EventArgs e)
        {
            fr_PatientSeeker fr = new fr_PatientSeeker();
            DialogResult dr = fr.ShowDialog();

            if (dr != DialogResult.OK) return;

            vPatientManager = fr.SelectedItem;
            tsMenu_SelectionMedic.Enabled = true;
            SetLabelsVisible(true);

            vPrescriptionList.Clear();
            dgv_Prescription.DataSource = null;

            #region Affichage Patient
            if (fr.SelectedItem.Sex == Sex.Homme)
                lbl_Nom.Text = "Monsieur ";
            else
                lbl_Nom.Text = "Madame ";

            lbl_Nom.Text += fr.SelectedItem.ToString();

            lbl_DateOfBirth.Text = "Date de naissance: " + fr.SelectedItem.DateOfBirth.ToLongDateString();
            lbl_Address.Text = "Adresse: " + fr.SelectedItem.Address;
            lbl_City.Text = "Ville: " + fr.SelectedItem.ZipCode + " " + fr.SelectedItem.City;
            #endregion

            #region Affichage Mdicale info
            int nbr =
                (int)SqlManager.ExecuteScalar("SELECT COUNT(*) FROM Prescription WHERE id_Patient = "
                + fr.SelectedItem.IDPatient);

            lbl_Prescriptions.Text = "Total des prescriptions: " + nbr;
            lbl_LastUpdate.Text = "Dernire visite: " + fr.SelectedItem.LastUpdate.ToShortDateString();
            lbl_Medic.Text = "Mdecin traitant: " + fr.SelectedItem.MedecinTraitant.ToString();
            lbl_Homeo.Text = "Homopathe: " + fr.SelectedItem.Homeopathe.ToString();

            #endregion
        }
        /// <summary>
        /// Selection d'un mdicament
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_SelectDrug(object sender, EventArgs e)
        {
            tb_Remarks.Text = "";
            fr_DrugsSeeker fr = new fr_DrugsSeeker();
            DialogResult dr = fr.ShowDialog();
            if (dr != DialogResult.OK) return;
            btn_Accepter.Enabled = true;
            tsMenu_AddPrescription.Enabled = true;
            vPrescriptionManager = new PrescriptionManager();
            vPrescriptionManager.PatientManager = vPatientManager;
            vPrescriptionManager.DrugsManager = fr.SelectedItem;
            vPrescriptionManager.Date = DateTime.Today;

            lbl_Medicament.Text = vPrescriptionManager.DrugsManager.ToString();
        }
        /// <summary>
        /// Fait la prescription
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_AddPrescription(object sender, EventArgs e)
        {
            vPrescriptionManager.Remark = tb_Remarks.Text;
            vPrescriptionManager.Add();
            vPrescriptionList.Add(vPrescriptionManager);

            dgv_Prescription.DataSource = null;
            dgv_Prescription.Rows.Clear();
            dgv_Prescription.DataSource = vPrescriptionList;

            for (int i = 0; i < dgv_Prescription.Columns.Count; i++)
                dgv_Prescription.Columns[i].Visible = false;

            dgv_Prescription.Columns["DrugsManager"].Visible = true;
            dgv_Prescription.Columns["DrugsManager"].DisplayIndex = 0;

            dgv_Prescription.Columns["Remark"].Visible = true;
            dgv_Prescription.Columns["Remark"].DisplayIndex = 1;

            lbl_Medicament.Text = "";
            btn_Accepter.Enabled = false;
            tsMenu_AddPrescription.Enabled = false;
        }
        /// <summary>
        /// Ajout d'un mdicament
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_AddDrug(object sender, EventArgs e)
        {
            fr_DrugsManager fr = new fr_DrugsManager();
            fr.Manager = new DrugsManager();
            fr.Use = Use.Add;
            fr.Size = new Size(600, fr.Size.Height);
            fr.ShowDialog();

        }
        /// <summary>
        /// Demande de modification d'un mdicament
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_ModifyDrug(object sender, EventArgs e)
        {
            fr_DrugsSeeker fr = new fr_DrugsSeeker();
            DialogResult dr = fr.ShowDialog();
            if (dr != DialogResult.OK) return;
            fr_DrugsManager dm = new fr_DrugsManager();
            dm.Use = Use.Update;
            dm.Size = new Size(600, dm.Size.Height);
            dm.Manager = fr.SelectedItem;
            dm.ShowDialog();
        }
        /// <summary>
        /// Demande d'ajout de patient
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_AddPatient(object sender, EventArgs e)
        {
            fr_PatientManager fr = new fr_PatientManager();
            fr.Size = new Size(fr.Size.Width, 600);
            fr.Use = Use.Add;
            fr.ShowDialog();
        }
        /// <summary>
        /// Modification d'un patient
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_ModifyPatient(object sender, EventArgs e)
        {
            fr_PatientSeeker ps = new fr_PatientSeeker();
            DialogResult dr = ps.ShowDialog();
            if (dr != DialogResult.OK) return;

            fr_PatientManager fr = new fr_PatientManager();
            fr.Manager = ps.SelectedItem;
            fr.Use = Use.Update;
            fr.Size = new Size(fr.Size.Width, 600);
            fr.ShowDialog();

        }
        /// <summary>
        /// Menu de demande d'impression
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_PrintPreview(object sender, EventArgs e)
        {
            vPrescriptionPrint.PrescriptionList = vPrescriptionList;
            vPrescriptionPrint.RefreshHeader();
            vPrescriptionPrint.Document.PrinterSettings = Options.Instance.PrinterSettingsPrescritions;
            ppd_PrintPreview.Document = vPrescriptionPrint.Document;
            ppd_PrintPreview.ShowDialog();
        }
        /// <summary>
        /// Prview de l'impression
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_SetupPrinter(object sender, EventArgs e)
        {
            PrintDialog pd = new PrintDialog();
            DialogResult dr = pd.ShowDialog();
            if (dr != DialogResult.OK) return;

            Options.Instance.PrinterSettingsPrescritions = pd.PrinterSettings;
            vPrescriptionPrint.PrescriptionList = vPrescriptionList;
            vPrescriptionPrint.Document.PrinterSettings = pd.PrinterSettings;

        }
        /// <summary>
        /// Evnement sur une demande de modification de prescription
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_ModificationPrescription(object sender, EventArgs e)
        {
            if (dgv_Prescription.Rows.Count < 1 || dgv_Prescription.SelectedRows.Count < 1) return;

            PrescriptionManager pm = vPrescriptionList[(int)dgv_Prescription.SelectedRows[0].Index];

            fr_PrescriptionManager fr = new fr_PrescriptionManager();
            fr.Manager = pm;
            fr.Use = Use.Update;
            fr.Text = "Modification d'une rputation";
            fr.Size = new Size(fr.Size.Width, 250);
            fr.ShowDialog();
        }
        /// <summary>
        /// Demande de suppression de la prescription
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_SuppressionPrescription(object sender, EventArgs e)
        {
            if (dgv_Prescription.Rows.Count < 1 || dgv_Prescription.SelectedRows.Count < 1) return;
            int idx = (int)dgv_Prescription.SelectedRows[0].Index;

            PrescriptionManager pm = vPrescriptionList[idx];
            DialogResult dr = MessageBox.Show("Voulez-vous supprimer cette prescription?\r\n"
                + vPrescriptionList[idx].ToString(), "Question", MessageBoxButtons.YesNo, MessageBoxIcon.Question);

            if (dr == DialogResult.Yes)
            {
                dgv_Prescription.DataSource = null;
                vPrescriptionList[idx].Delete();
                vPrescriptionList.RemoveAt(idx);
                dgv_Prescription.DataSource = vPrescriptionList;

                for (int i = 0; i < dgv_Prescription.Columns.Count; i++)
                    dgv_Prescription.Columns[i].Visible = false;

                dgv_Prescription.Columns["DrugsManager"].Visible = true;
                dgv_Prescription.Columns["DrugsManager"].DisplayIndex = 0;

                dgv_Prescription.Columns["Remark"].Visible = true;
                dgv_Prescription.Columns["Remark"].DisplayIndex = 1;
            }
        }
        /// <summary>
        /// A propos
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_APropos(object sender, EventArgs e)
        {
            fr_Main fr = new fr_Main();
            fr.ShowDialog();
        }
        /// <summary>
        /// Donne l'historique d'un patient
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_HistoriquePatient(object sender, EventArgs e)
        {
            fr_PatientSeeker fr = new fr_PatientSeeker();
            DialogResult dr = fr.ShowDialog();

            if (dr != DialogResult.OK) return;

            fr_HistoriquePatient hp = new fr_HistoriquePatient();
            hp.PatientManager = fr.SelectedItem;
            hp.ShowDialog();

            //We deleted prescriptions. Refresh needed
            if (hp.Deleted)
            {
                PrescriptionManager p = new PrescriptionManager();
                for (int i = 0; i < vPrescriptionList.Count; i++)
                {
                    p = PrescriptionManager.Create(vPrescriptionList[i].IDPrescription);
                    if (p.Empty)
                        vPrescriptionList.Remove(vPrescriptionList[i]);
                }

                if (vPrescriptionList.Count > 0)
                {
                    dgv_Prescription.DataSource = null;
                    dgv_Prescription.DataSource = vPrescriptionList;

                    for (int i = 0; i < dgv_Prescription.Columns.Count; i++)
                        dgv_Prescription.Columns[i].Visible = false;

                    dgv_Prescription.Columns["DrugsManager"].Visible = true;
                    dgv_Prescription.Columns["DrugsManager"].DisplayIndex = 0;

                    dgv_Prescription.Columns["Remark"].Visible = true;
                    dgv_Prescription.Columns["Remark"].DisplayIndex = 1;
                }
                else
                {
                    //dgv_Prescription.Rows.Clear();
                    dgv_Prescription.DataSource = null;
                }
            }
        }
        /// <summary>
        /// Demande d'une suppression de mdicament
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_DelDrugType(object sender, EventArgs e)
        {
            try
            {
                fr_DrugsTypeSeeker dts = new fr_DrugsTypeSeeker();
                dts.btn_AddReputation.Visible = false;
                DialogResult dr = dts.ShowDialog();
                if (dr == DialogResult.Cancel) return;
                DrugsTypeManager dtm = dts.SelectedItem;
                dtm.Delete();
            }
            catch (OleDbException ex)
            {
                MessageBox.Show("Impossible de supprimer ce type de mdicament\r\n" + ex.Message, "Attention",
                     MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
            }
            catch (Exception ex)
            {
                MessageBox.Show("Une erreur est survenue pendant la suppression\r\n" + ex.Message, "Erreur",
                    MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }
        /// <summary>
        /// Clique sur la gestion d'affichage des menus
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_Managment(object sender, EventArgs e)
        {
            if (!tsMenu_Managment.Checked)
            {
                tsMenu_Managment.Checked = true;
                ts_Managment.Visible = true;
            }
            else
            {
                tsMenu_Managment.Checked = false;
                ts_Managment.Visible = false;
            }
            PegaMenu p = new PegaMenu();
            p.ManagmentVisible = ts_Managment.Visible;
            Options.Instance.PegaMenu = p;
        }
        #endregion

        #region Mthodes
        /// <summary>
        /// Rends les diffrents labels de la fentre visible ou pas
        /// </summary>
        /// <param name="_IsVisible">A true tout est visible,  false tout est invisible</param>
        private void SetLabelsVisible(bool _IsVisible)
        {
            #region Misc
            btn_SelectMedic.Enabled = _IsVisible;
            tsMenu_SelectionMedic.Enabled = _IsVisible;
            #endregion

            lbl_Address.Visible = _IsVisible;
            lbl_City.Visible = _IsVisible;
            lbl_DateOfBirth.Visible = _IsVisible;
            lbl_Homeo.Visible = _IsVisible;
            lbl_LastUpdate.Visible = _IsVisible; ;
            lbl_Medic.Visible = _IsVisible;
            lbl_Medicament.Visible = _IsVisible;
            lbl_Nom.Visible = _IsVisible;
            lbl_Prescriptions.Visible = _IsVisible;
        }
        #endregion
    }
}